@@ -1,67 +1,93 @@
-import importlib
-import sys
+# **********************************************************************************************************************
+# SpartaQuant is a modular platform both for Python developers and decision makers
+# This API requires an access token that can be generated in your SpartaQuant settings section
+# Make sure you already have an account or get one at https://www.spartaquant.com
+# For more information regarding this API, check the webpage https://spartaquant.pro/publicAPI
+
+# Author: Benjamin Meyer
+# **********************************************************************************************************************
+
 import os
 import getpass
-
 import requests
 import inspect
 import json
 import base64
-import cloudpickle
+import pickle
 import sqlite3
 
-
 class spartaquant():
+    '''
+        To use this API you can start with the following lines:
 
-    def __init__(self):
+        import spartaquant as sq
+        sq = sq.spartaquant()
+
+        Then, you can retrieve information using the following method, for instance
+            sq.getDataDB() to get all the available SpartaQuant objects
+            sq.getFunctionsDB() to get all the available SpartaQuant functions
+            etc...
+    '''
+
+    def __init__(self, bAuthenticate=True):
         self.URL_BASE = None
-        self.USER_ID = None
-        self.authenticate()
+        self.VALID_ACCESS_KEY = None
+        if bAuthenticate:
+            self.authenticate()
 
     def authenticate(self):
-        # **************************************************************************************************************
+        '''
+            API authentication
+        '''
         bInternalJupyterDesktop = False
         # try:
         #     self.URL_BASE, self.USER_ID = self.getDBAuthFunc()
         #     bInternalJupyterDesktop = True
         # except Exception as e:
         #     pass
-        
+        if not bInternalJupyterDesktop:
+            accessKey = getpass.getpass(prompt="Enter your access key")
+            return self.authenticateFunc(accessKey)
+
+    def authenticateFunc(self, accessKey):
+        '''
+            API authentication function
+        '''
         def printError():
             print("Authentication failed. Make sure you have entered the correct accessKey")
 
-        if not bInternalJupyterDesktop:
-            accessKey = getpass.getpass(prompt="Enter your access key")
-            try:
-                mainUrl = self.decodeMainUrl(accessKey)
-            except:
-                printError()
-                return
-                
-            validateUrl = mainUrl+'validateMainUrlExternalApi'
-            newJson = dict()
-            json_data = dict()
-            json_data['accessKey'] = accessKey
-            newJson['jsonData'] = json.dumps(json_data)
-            newJsonB = json.dumps(newJson)
-            try:
-                res = requests.post(validateUrl, data=newJsonB, verify=False)
-                statusCode = res.status_code
-                if int(statusCode) == 200:
-                    resJson = json.loads(res.text)
-                    res = resJson['res']
-                    if res == 1:
-                        self.URL_BASE = mainUrl
-                        self.USER_ID = self.decodeb64(self.decodeb64(resJson['userId']))
-                        print("You are logged "+str(chr(0x2713)))
-                    else:
-                        printError()
+        try:
+            mainUrl = self.decodeMainUrl(accessKey)
+        except:
+            printError()
+            return
+            
+        validateUrl = mainUrl+'validateMainUrlExternalApi'
+        newJson = dict()
+        json_data = dict()
+        json_data['accessKey'] = accessKey
+        newJson['jsonData'] = json.dumps(json_data)
+        newJsonB = json.dumps(newJson)
+        try:
+            res = requests.post(validateUrl, data=newJsonB, verify=False)
+            statusCode = res.status_code
+            if int(statusCode) == 200:
+                resJson = json.loads(res.text)
+                res = resJson['res']
+                if res == 1:
+                    self.VALID_ACCESS_KEY = accessKey
+                    self.URL_BASE = self.decodeMainUrl(accessKey)
+                    print("You are logged "+str(chr(0x2713)))
                 else:
                     printError()
-            except:
+            else:
                 printError()
-
-        # **************************************************************************************************************
+        except:
+            printError()
+        
+    def authenticateAPI(self, accessKey):
+        self.VALID_ACCESS_KEY = accessKey
+        self.URL_BASE = self.decodeMainUrl(accessKey)
 
     def decodeMainUrl(self, accessKey):
         return self.decodeFunc(self.decodeFunc(accessKey).split('__sq__')[0])
@@ -91,53 +117,47 @@ class spartaquant():
         except:
             return None
 
-    # def getDBAuth(self):
-    #     global bInternalJupyterDesktop
-    #     if bInternalJupyterDesktop:
-    #         return getDBAuthFunc()
-    #     else:
-    #         global urlExternalApi
-    #         global userIdExternalApi
-    #         return urlExternalApi, userIdExternalApi
-
     def sendRequests(self, funcName, *args):
-        # try:
-        #     URL_BASE, USER_ID = self.getDBAuth()
-        # except:
-        #     print("Authentication failed. You may need to restart the kernel")
-        #     return 
-        if self.URL_BASE is None or self.USER_ID is None:
-            print("Authentication failed. You may need to reload the SpartaQuant module")
+        if self.URL_BASE is None or self.VALID_ACCESS_KEY is None:
+            print("Authentication failed. You may need to re-import the SpartaQuant module")
 
         thisUrl = self.URL_BASE+"jupyterAPI"
         newJson = dict()
-        json_data = dict()
         argsSerialized = []
         for thisArg in args:
-            data_bin = cloudpickle.dumps(thisArg)
+            data_bin = pickle.dumps(thisArg)
             serializedObj = str(base64.b64encode(data_bin), "utf-8")
             argsSerialized.append(serializedObj)
         
-        json_data['userId'] = 'EXT_API'+str(base64.b64encode(str(self.USER_ID).encode()), 'utf-8')
-        json_data['funcName'] = funcName
-        json_data['args'] = argsSerialized
-        newJson['jsonData'] = json.dumps(json_data)
+        newJson['jsonData'] = json.dumps({
+            'accessKey': self.VALID_ACCESS_KEY,
+            'funcName': funcName,
+            'args': argsSerialized
+        })
         newJsonB = json.dumps(newJson)
         res = requests.post(thisUrl, data=newJsonB, verify=False)
         resJson = json.loads(res.text)
         if int(resJson['res']) == 1:
-            return cloudpickle.loads(base64.b64decode(resJson['serializedObj']))
+            return pickle.loads(base64.b64decode(resJson['serializedObj']))
         else:
             print("Could not proceed the request")
             return {'res': -1}
 
+    def getUUID(self):
+        funcName = str(inspect.stack()[0][0].f_code.co_name)
+        return self.sendRequests(funcName)
+
     def getDataDB(self):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
         return self.sendRequests(funcName)
 
-    def getData(self, thisData):
+    def getData(self, apiId, dispoDate=None):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
-        return self.sendRequests(funcName, thisData)
+        return self.sendRequests(funcName, apiId, dispoDate)
+
+    def getDispoDates(self, apiId):
+        funcName = str(inspect.stack()[0][0].f_code.co_name)
+        return self.sendRequests(funcName, apiId)
 
     def getDataDates(self, thisDates, formula, bBusiness=True, formatDate='%Y%m%d'):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
@@ -151,17 +171,17 @@ class spartaquant():
         funcName = str(inspect.stack()[0][0].f_code.co_name)
         return self.sendRequests(funcName)
 
-    def getMTD(self):
+    def getMTD(self, thisDate=None, formatDate="%Y%m%d"):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
-        return self.sendRequests(funcName)
+        return self.sendRequests(funcName, thisDate, formatDate)
 
-    def getQTD(self):
+    def getQTD(self, thisDate=None, formatDate="%Y%m%d"):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
-        return self.sendRequests(funcName)
+        return self.sendRequests(funcName, thisDate, formatDate)
 
-    def getYTD(self):
+    def getYTD(self, thisDate=None, formatDate="%Y%m%d"):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
-        return self.sendRequests(funcName)
+        return self.sendRequests(funcName, thisDate, formatDate)
 
     def putExec(self, str2Eval, name):
         funcName = str(inspect.stack()[0][0].f_code.co_name)
@@ -230,5 +250,4 @@ class spartaquant():
         # functionSource = inspect.getsource(functionObj)
         # funcName = str(inspect.stack()[0][0].f_code.co_name)
         return "Function tested"
-
-    
\ No newline at end of file
+   
\ No newline at end of file
